@@ -41,6 +41,8 @@ static GObjectClass *parent_class = NULL;
 /* Acquisition thread */
 
 typedef struct {
+	ArvStream *stream;
+
 	ArvStreamCallback callback;
 	void *user_data;
 
@@ -52,8 +54,6 @@ typedef struct {
 	guint64 timestamp_tick_frequency;
 
 	gboolean cancel;
-	GAsyncQueue *input_queue;
-	GAsyncQueue *output_queue;
 
 	guint32 packet_count;
 
@@ -83,7 +83,7 @@ _send_packet_request (ArvGvStreamThreadData *thread_data,
 		      guint32 last_block)
 {
 	ArvGvcpPacket *packet;
-	guint32 packet_size;
+	size_t packet_size;
 
 	packet = arv_gvcp_packet_new_packet_resend_cmd (frame_id, first_block, last_block,
 							thread_data->packet_count++, &packet_size);
@@ -141,6 +141,7 @@ typedef struct {
 	gint64 last_time_us;
 	guint64 last_timestamp_ns;
 	guint32 statistic_count;
+	gint64 leader_time_us;
 } ArvGvStreamThreadState;
 
 static void
@@ -180,27 +181,34 @@ _close_buffer (ArvGvStreamThreadData *thread_data, ArvGvStreamThreadState *state
 	current_time_us = current_time.tv_sec * 1000000 + current_time.tv_usec;
 	if (state->statistic_count > 5) {
 		arv_statistic_fill (thread_data->statistic, 0,
-				    (current_time_us - state->last_time_us),
-				    state->buffer->frame_id);
-		arv_statistic_fill (thread_data->statistic, 1,
 				    (state->buffer->timestamp_ns - state->last_timestamp_ns) /
 				    1000, state->buffer->frame_id);
+		arv_statistic_fill (thread_data->statistic, 1,
+				    current_time_us - state->last_time_us,
+				    state->buffer->frame_id);
+		arv_statistic_fill (thread_data->statistic, 2,
+				    current_time_us - state->leader_time_us,
+				    state->buffer->frame_id);
 	} else
 		state->statistic_count++;
 
 	state->last_time_us = current_time_us;
 	state->last_timestamp_ns = state->buffer->timestamp_ns;
-	g_async_queue_push (thread_data->output_queue, state->buffer);
+
+	arv_stream_push_output_buffer (thread_data->stream, state->buffer);
+
 	state->buffer = NULL;
 }
 
 static void
 _process_data_leader (ArvGvStreamThreadData *thread_data, ArvGvStreamThreadState *state)
 {
+	GTimeVal current_time;
+
 	if (state->buffer != NULL)
 		_close_buffer (thread_data, state);
 
-	state->buffer = g_async_queue_try_pop (thread_data->input_queue);
+	state->buffer = arv_stream_pop_input_buffer (thread_data->stream);
 	if (state->buffer == NULL) {
 		thread_data->n_underruns++;
 		return;
@@ -223,6 +231,14 @@ _process_data_leader (ArvGvStreamThreadData *thread_data, ArvGvStreamThreadState
 	state->n_missing_blocks = 0;
 	state->last_block_size = 0;
 	state->last_block_id = 0;
+
+	g_get_current_time (&current_time);
+	state->leader_time_us = current_time.tv_sec * 1000000 + current_time.tv_usec;
+
+	if (thread_data->callback != NULL)
+		thread_data->callback (thread_data->user_data,
+				       ARV_STREAM_CALLBACK_TYPE_START_BUFFER,
+				       state->buffer);
 }
 
 static void
@@ -401,7 +417,7 @@ arv_gv_stream_set_option (ArvGvStream *gv_stream, ArvGvStreamOption option, int
 			break;
 		case ARV_GV_STREAM_OPTION_SOCKET_BUFFER_FIXED:
 			thread_data->socket_buffer_option = option;
-			thread_data->socket_buffer_size = 0;
+			thread_data->socket_buffer_size = value;
 			break;
 	}
 }
@@ -443,6 +459,7 @@ arv_gv_stream_new (GInetAddress *device_address, guint16 port,
 	g_socket_bind (gv_stream->socket, gv_stream->incoming_address, TRUE, NULL);
 
 	thread_data = g_new (ArvGvStreamThreadData, 1);
+	thread_data->stream = stream;
 	thread_data->callback = callback;
 	thread_data->user_data = user_data;
 	thread_data->socket = gv_stream->socket;
@@ -450,8 +467,6 @@ arv_gv_stream_new (GInetAddress *device_address, guint16 port,
 	thread_data->packet_resend = ARV_GV_STREAM_PACKET_RESEND_ALWAYS;
 	thread_data->timestamp_tick_frequency = timestamp_tick_frequency;
 	thread_data->cancel = FALSE;
-	thread_data->input_queue = stream->input_queue;
-	thread_data->output_queue = stream->output_queue;
 
 	thread_data->packet_count = 1;
 
@@ -464,10 +479,11 @@ arv_gv_stream_new (GInetAddress *device_address, guint16 port,
 	thread_data->n_late_blocks = 0;
 	thread_data->n_aborteds = 0;
 
-	thread_data->statistic = arv_statistic_new (2, 5000, 200, 0);
+	thread_data->statistic = arv_statistic_new (3, 5000, 200, 0);
 
-	arv_statistic_set_name (thread_data->statistic, 0, "Local time delta");
-	arv_statistic_set_name (thread_data->statistic, 1, "Timestamp delta");
+	arv_statistic_set_name (thread_data->statistic, 0, "Timestamp delta");
+	arv_statistic_set_name (thread_data->statistic, 1, "Local time delta");
+	arv_statistic_set_name (thread_data->statistic, 2, "Buffer reception time");
 
 	thread_data->socket_buffer_option = ARV_GV_STREAM_OPTION_SOCKET_BUFFER_FIXED;
 	thread_data->socket_buffer_size = 0;
@@ -482,11 +498,32 @@ arv_gv_stream_new (GInetAddress *device_address, guint16 port,
 
 /* ArvStream implementation */
 
+void
+arv_gv_stream_get_statistics (ArvGvStream *gv_stream,
+			      guint64 *n_resent_blocks,
+			      guint64 *n_missing_blocks,
+			      guint64 *n_late_blocks)
+
+{
+	ArvGvStreamThreadData *thread_data;
+
+	g_return_if_fail (ARV_IS_GV_STREAM (gv_stream));
+
+	thread_data = gv_stream->thread_data;
+
+	if (n_resent_blocks != NULL)
+		*n_resent_blocks = thread_data->n_resent_blocks;
+	if (n_missing_blocks != NULL)
+		*n_missing_blocks = thread_data->n_missing_blocks;
+	if (n_late_blocks != NULL)
+		*n_late_blocks = thread_data->n_late_blocks;
+}
+
 static void
-arv_gv_stream_get_statistics (ArvStream *stream,
-			      guint64 *n_completed_buffers,
-			      guint64 *n_failures,
-			      guint64 *n_underruns)
+_get_statistics (ArvStream *stream,
+		 guint64 *n_completed_buffers,
+		 guint64 *n_failures,
+		 guint64 *n_underruns)
 {
 	ArvGvStream *gv_stream = ARV_GV_STREAM (stream);
 	ArvGvStreamThreadData *thread_data;
@@ -564,7 +601,7 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 
 	object_class->finalize = arv_gv_stream_finalize;
 
-	stream_class->get_statistics = arv_gv_stream_get_statistics;
+	stream_class->get_statistics = _get_statistics;
 }
 
 G_DEFINE_TYPE (ArvGvStream, arv_gv_stream, ARV_TYPE_STREAM)
