@@ -27,7 +27,14 @@
 
 G_BEGIN_DECLS
 
-ArvDevice * 		arv_new_device 		(const char *name);
+unsigned int 		arv_get_n_interfaces 		(void);
+const char * 		arv_get_interface_id 		(unsigned int index);
+
+void 			arv_update_device_list 		(void);
+unsigned int 		arv_get_n_devices 		(void);
+const char * 		arv_get_device_id 		(unsigned int index);
+
+ArvDevice * 		arv_open_device 		(const char *device_id);
 
 G_END_DECLS
 
